#! /bin/bash
# -----------------------------------------------------------------------------------------------------------
# Tuxbox Build & Compiler Helper
# by yjogol (yjogol@online.de)
# $Date: 2007/02/09 16:25:40 $
# $Revision: 1.11 $
# -----------------------------------------------------------------------------------------------------------

# -----------------------------------------------------------------------------------------------------------
# Globals
# -----------------------------------------------------------------------------------------------------------
if [ -e ./yb_globals.conf ]; then
	SCRIPTDIR=`pwd`
else
	SCRIPTDIR=$HOME/tuxbox/yBuild
fi
yb_configfile=$SCRIPTDIR/yb_globals.conf
yb_versionfile=$SCRIPTDIR/yb_version.txt
LOGFILE=$SCRIPTDIR/build.log
SDEBUG=${1:-0} # use yb_start 1 to start in debug mode
DEBUG=0
complete_diff="all.diff"
if [ "`which ccache`" == "" ]; then
	HAVE_CCACHE="ccache not installed"
else
	HAVE_CCACHE="ccache installed"
fi
# -----------------------------------------------------------------------------------------------------------
# Inculde Settings
# -----------------------------------------------------------------------------------------------------------
. $yb_versionfile
. $yb_configfile
. $SCRIPTDIR/include/_yb_library.inc.sh
. $SCRIPTDIR/include/_yb_basisconf.inc.sh
. $SCRIPTDIR/include/_yb_buildconf.inc.sh
. $SCRIPTDIR/include/_yb_make.inc.sh
. $SCRIPTDIR/include/_yb_configureconf.inc.sh
. $SCRIPTDIR/include/_yb_cleantargets.inc.sh
. $SCRIPTDIR/include/_yb_maketargets.inc.sh

# -----------------------------------------------------------------------------------------------------------
# Main Menu
# -----------------------------------------------------------------------------------------------------------
menu_start()
{
	if [ "$SDEBUG" == "0" ];then
		clear
	fi
	echo "================================================================================"
	echo "Tuxbox - Build & Compiling Helper"
	echo "Version: $yb_version Date: $yb_date"
	echo "--------------------------------------------------------------------------------"
	echo "0   + Basis Configuration"
	echo "cc  + Configure Configuration"
	echo "bc  + Build Configuration"
	echo "--------------------------------------------------------------------------------"
	echo "go  - Build an image in one step with the selected configurations"
	echo "      --------------------------------------------------------------------------"
	echo "      --------------------------------------------------------------------------"
	echo "co  - Checkout  - First time / complete"
	echo "cu  - Checkout  - update"
	echo "      --------------------------------------------------------------------------"
	echo "cf  - Configure - Flash"
	echo "cy  - Configure - Yadd"
	echo "p   - Patch manually (from $MyPATCHESDIR)"
	echo "      --------------------------------------------------------------------------"
	echo "bs  - Build Selected ( make $cBuildTARGET-$cBuildGUI-$cBuildFS-$cBuildCHIPS )"
	echo "bf  - Build Flash All ( make flash all )"
	echo "by  - Build Yadd All ( make yadd all )"
	echo "      --------------------------------------------------------------------------"
	echo "ct  + Clean Targets" 
	echo "mt  + Make Targets" 
	echo "      --------------------------------------------------------------------------"
	echo "cd  - Create a complete diff against cvs (save to $MyPATCHESDIR/$complete_diff)"
	echo "m   + Make Menu : Will ask you"
	echo "d   - Debug on=1/off=0 : $DEBUG (make & CVS will be simulated)"
	echo "      --------------------------------------------------------------------------"
	echo "t   - Toolchecker"
	echo "l   - View Logfile"
	echo "      --------------------------------------------------------------------------"
	echo "d*  - dhcp *=0:off/*=1:on (disable/enable YADD booting)"
	echo "ccc - Clear Compiler cache ($HAVE_CCACHE)"
	echo "dis - Build a distribution of your changed yBuild in ./distribution/yBuild.zip"
	echo "upd - Download the actual yBuild.zip from yjogol.de. Overrides your scripts!"
	echo ""
	echo "x   - Quit"
	echo "==============================================================================="
	echo "Enter command:"
}

# -----------------------------------------------------------------------------------------------------------
# Clean Directoty Structure
# -----------------------------------------------------------------------------------------------------------
clean_dirs()
{
	rm -frdv $CVSDIR/*
	rm -frdv $DBOX_PREFIX/cdk
	rm -frdv $DBOX_PREFIX/cdkflash
	rm -frdv $DBOX_PREFIX/cdkroot
}

# -----------------------------------------------------------------------------------------------------------
# Build Directoty Structure if not exists
# -----------------------------------------------------------------------------------------------------------
build_dirs()
{
	mkdir -p $DBOX_PREFIX
	mkdir -p $CVSDIR
	mkdir -p $LOGOSDIR
	mkdir -p $UCODESDIR
	mkdir -p $ARCHIVEDIR 
	mkdir -p $MyLOCALSDIR
	mkdir -p $MyPATCHESDIR
	mkdir -p $MyDBOXFILESDIR
}

# -----------------------------------------------------------------------------------------------------------
# Build Configure String for Options
# $1=var-prefix $2=FLASH|YADD $3=feature
# -----------------------------------------------------------------------------------------------------------
_do_configure_build_features()
{
	varname="conf$1"_"$2"
	val=${!varname}
	if [ "$val" == "y" ]; then
		flag="--enable-$3"
	else
		flag="--disable-$3"
	fi
}
# -----------------------------------------------------------------------------------------------------------
# Build Configure String for Options
# $1=FLASH | YADD
# -----------------------------------------------------------------------------------------------------------
do_configure_build_features()
{
	flags=""
	for f in "ide" "lirc" "ext3" "xfs" "nfsserver" "sambaserver" "cdkVcInfo" "dosfstools" "upnp"
	do
		_do_configure_build_features $f $1 $f
		flags="$flags $flag"
	done
	_do_configure_build_features germankeymaps $1 "german-keymaps"
	flags="$flags $flag"
}

# -----------------------------------------------------------------------------------------------------------
# Configure Flash
# -----------------------------------------------------------------------------------------------------------
do_configure_flash()
{
	echo "=============================================================="
	echo "configure"
	echo "=============================================================="
	if [ "$confautopatch_FLASH" == "y" ];then
		echo "Do you really want to patch the cvs and then configure? (y/n)"
	else	
		echo "Do you really want to configure? (y/n)"
	fi
	read yesno
	if [ "$yesno" != "n" ];then
		if [ "$confautopatch_FLASH" == "y" ];then
			do_patch
		fi
		init_variables
		cd "$CVSDIR/cdk"
		/bin/ln -sf $ARCHIVEDIR $CVSDIR/cdk/Archive
		./autogen.sh

		# build configure parameters for settings
		do_configure_build_features FLASH
		if [ "$ROOTPARTSIZE" != "" ]; then
			flags="$flags --with-rootpartitionsize=$ROOTPARTSIZE"
		fi
		if [ "$UPDATEHTTPPREFIX" != "" ]; then
			flags="$flags --with-updatehttpprefix=$UPDATEHTTPPREFIX"
		fi
		echo "FLAGS generated = $flags"
		./configure --prefix="$DBOX_PREFIX" --with-cvsdir="$CVSDIR" --enable-flashrules \
			--with-logosdir="$LOGOSDIR" \
			--with-ucodesdir=$UCODESDIR \
			--with-checkImage=rename \
			--enable-maintainer-mode \
			--with-customizationsdir="$MyLOCALSDIR" \
			$flags
	fi
}

# -----------------------------------------------------------------------------------------------------------
# Configure YADD
# -----------------------------------------------------------------------------------------------------------
do_configure_yadd()
{
	echo "=============================================================="
	echo "configure YADD"
	echo "=============================================================="
	if [ "$confautopatch_YADD" == "y" ];then
		echo "Do you really want to patch the cvs and then configure? (y/n)"
	else
		echo "Do you really want to configure? (y/n)"
	fi
	read yesno
	if [ "$yesno" != "n" ];then
		if [ "$confautopatch_YADD" == "y" ];then
			do_patch
		fi
		init_variables
		cd "$CVSDIR/cdk"
		/bin/ln -sf $ARCHIVEDIR $CVSDIR/cdk/Archive
		./autogen.sh
		# build configure parameters for settings
		do_configure_build_features YADD
		if [ "$UPDATEHTTPPREFIX" != "" ]; then
			flags="$flags --with-updatehttpprefix=$UPDATEHTTPPREFIX"
		fi
		echo "FLAGS generated = $flags"
		./configure --prefix="$DBOX_PREFIX" --with-cvsdir="$CVSDIR" \
			--with-logosdir="$LOGOSDIR" \
			--with-ucodesdir=$UCODESDIR \
			--enable-maintainer-mode \
			--with-customizationsdir="$MyLOCALSDIR" \
			$flags
	fi
}

# -----------------------------------------------------------------------------------------------------------
# CVS Checkout all
# first checkout "HEAD" then checkout newmake Makefiles
# So the CVS directories have the right tag. You will need this, if you have write access to cvs.
# -----------------------------------------------------------------------------------------------------------
cvs_checkout_all()
{
	build_dirs
	cvs_debug=""
	if [ "$DEBUG" == "1" ];then
		cvs_debug="-n -t"
	fi
	echo "=============================================================="
	echo "Start CVS Checkout to $CVSDIR (DEBUG=$DEBUG)"
	echo "=============================================================="
	echo "Do you really want to checkout ALL? (y/n)"
	read yesno
	if [ "$yesno" == "y" ];then
		if [ "$CVSNAME" == "anoncvs" ];then
		# Checkout if CVS-User is anonymous
			echo "--------------------------------------------------------------"
			echo "Checkout newmake anonymous"
			echo "--------------------------------------------------------------"
			cd "$CVSDIR"
			set CVS_RSH=ssh && cvs $cvs_debug -d anoncvs@cvs.tuxbox.org:/cvs/tuxbox -z3 co -f -r newmake -P .
		else
		# Checkout if CVS-User is a registred user
			cd "$CVSDIR"
			echo "--------------------------------------------------------------"
			echo "Checkout HEAD registred"
			echo "--------------------------------------------------------------"
			cvs -z3 $cvs_debug -d "$CVSNAME@cvs.tuxbox.org:/cvs/tuxbox" co -P .
			if ! [ -d "$CVSDIR/cdk" ];then
				echo "No cdk dir found in $CVSDIR. STOP"
				exit
			fi
			echo "--------------------------------------------------------------"
			echo "Checkout newmake registred"
			echo "--------------------------------------------------------------"
			cd "$CVSDIR"
			cvs -z3 $cvs_debug -d "$CVSNAME@cvs.tuxbox.org:/cvs/tuxbox" co -r newmake -P cdk/newmake.files
			cd "$CVSDIR"
			cvs -z3 $cvs_debug -d "$CVSNAME@cvs.tuxbox.org:/cvs/tuxbox" co -f -r newmake `cat cdk/newmake.files`
		fi
		rm $CVSDIR/cdk/root/etc/init.d/rcS $CVSDIR/cdk/root/etc/init.d/rcS.insmod
	fi
}

# -----------------------------------------------------------------------------------------------------------
# CVS Checkout Update
# -----------------------------------------------------------------------------------------------------------
cvs_checkout_update()
{
	cvs_debug=""
	if [ "$DEBUG" == "1" ]; then
		cvs_debug="-n -t"
	fi
	echo "=============================================================="
	echo "Start CVS Checkout UPDATE to $CVSDIR (DEBUG=$DEBUG)"
	echo "=============================================================="
	echo "Do you really want to checkout UPDATE? (y/n)"
	read yesno
	if [ "$yesno" == "y" ];then
		if [ "$CVSNAME" == "anoncvs" ];then
		# Checkout if CVS-User is anonymous
			echo "--------------------------------------------------------------"
			echo "Checkout newmake anonymous"
			echo "--------------------------------------------------------------"
			cd "$CVSDIR"
			set CVS_RSH=ssh && cvs $cvs_debug -d anoncvs@cvs.tuxbox.org:/cvs/tuxbox -z3 co -f -r newmake -P .
		else
		# Checkout if CVS-User is a registred user
			cd "$CVSDIR"
			echo "--------------------------------------------------------------"
			echo "Checkout HEAD registred"
			echo "--------------------------------------------------------------"
			cvs -z3 $cvs_debug -d "$CVSNAME@cvs.tuxbox.org:/cvs/tuxbox" up -dP 
			echo "--------------------------------------------------------------"
			echo "Checkout newmake registred"
			echo "--------------------------------------------------------------"
			cd "$CVSDIR"
			cvs -z3 $cvs_debug -d "$CVSNAME@cvs.tuxbox.org:/cvs/tuxbox" co -r newmake -P cdk/newmake.files
			cd "$CVSDIR"
			cvs -z3 $cvs_debug -d "$CVSNAME@cvs.tuxbox.org:/cvs/tuxbox" co -f -r newmake `cat cdk/newmake.files`
		fi
		rm $CVSDIR/cdk/root/etc/init.d/rcS $CVSDIR/cdk/root/etc/init.d/rcS.insmod
	fi
}

# -----------------------------------------------------------------------------------------------------------
# execute MAKE $1=target $2=options
# enable logfile
# -----------------------------------------------------------------------------------------------------------
do_make()
{
	_do_make $* | tee $LOGFILE
}
# -----------------------------------------------------------------------------------------------------------
_do_make()
{
	echo "=============================================================="
	echo "Make $2 $1"
	date
	echo "=============================================================="
	cd $CVSDIR/cdk
	if [ "$DEBUG" == "1" ];then
		echo "--------------------------------------------------------------"
		echo " DEBUG is ON - MAKE is only simulated - nothing really done"
		echo "--------------------------------------------------------------"
		make -n $2 $1
		echo "--------------------------------------------------------------"
		echo " DEBUG is ON - MAKE is only simulated - nothing really done"
		echo "--------------------------------------------------------------"
	else
		make $2 $1
	fi
}

# -----------------------------------------------------------------------------------------------------------
# Build $1=Target $2=GUI, $3=Filesystem, $4=Chips
# enable logfile
# -----------------------------------------------------------------------------------------------------------
do_build()
{
	_do_build $* | tee $LOGFILE
}
# make
# -----------------------------------------------------------------------------------------------------------
_do_build()
{
	echo "=============================================================="
	echo "Build $1 / $2 / $3 / $4"
	date
	echo "=============================================================="
	cd "$CVSDIR/cdk"
	if [ "$1" == "flash" ]; then
		make $1-$2-$3-$4
	elif [ "$1" == "yadd" ]; then
		make $1-$2
  fi
}

# -----------------------------------------------------------------------------------------------------------
# do patch
# -----------------------------------------------------------------------------------------------------------
do_patch()
{
	_do_patch $* | tee $LOGFILE
}
# -----------------------------------------------------------------------------------------------------------
_do_patch()
{
	build_dirs
	echo "=============================================================="
	echo "Patching"
	echo "=============================================================="
	patchlist=`cat $MyPATCHESDIR/patches.txt`
	echo "$patchlist"
	mkdir -p $MyPATCHESDIR/backup
	for i in $patchlist
	do
		if [ "$i" != "" ]; then
			
			patch_file=`echo $i|cut -d ";" -f 2`
			patch_path=`echo $i|cut -d ";" -f 1`
			patch_num_slashes=`echo $i|cut -d ";" -f 3`
			if [ "$patch_num_slashes" == "" ]; then
				patch_num_slashes="0"
			fi
			echo "patch $patch_file at path: $patch_path slashes: $patch_num_slashes"
			patch -d $CVSDIR/$patch_path -b -B $MyPATCHESDIR/backup/bak- -p$patch_num_slashes <$MyPATCHESDIR/$patch_file
		fi
	done
	cp -f $MyPATCHESDIR/patches.txt $MyPATCHESDIR/backup/done_patches.txt
	cd $SCRIPTDIR
}

# -----------------------------------------------------------------------------------------------------------
# delete previously patched files from local cvs
# -----------------------------------------------------------------------------------------------------------
do_delete_files()
{
	_do_delete_files $* | tee $LOGFILE
}
# -----------------------------------------------------------------------------------------------------------
_do_delete_files()
{
	if [ -e $MyPATCHESDIR/backup/done_patches.txt ];then
		echo "=============================================================="
		echo "delete previously patched files from local cvs"
		echo "=============================================================="
		patchlist=`cat $MyPATCHESDIR/backup/done_patches.txt`
		for i in $patchlist
		do
			if [ "$i" != "" ]; then
				patch_path=`echo $i|sed -e "s/;.*$//g"`
				patch_file=`echo $i|sed -e "s/^.*;//;s/.diff.*$//g"`
				echo "delete: $CVSDIR/$patch_path/$patch_file"
				rm -f $CVSDIR/$patch_path/$patch_file*
			fi
		done
		rm -rf $MyPATCHESDIR/backup
		echo "Ready .... press Enter"
		read dummy
		cd $SCRIPTDIR
	fi
}

# -----------------------------------------------------------------------------------------------------------
# create complete diff
# -----------------------------------------------------------------------------------------------------------
do_create_diff()
{
	cvs_debug=""
	if [ "$DEBUG" == "1" ]; then
		cvs_debug="-n -t"
	fi
	echo "=============================================================="
	echo "Create complete diff to $MyPATCHESDIR/$complete_diff (DEBUG=$DEBUG)"
	echo "=============================================================="
	
	if [ "$CVSNAME" == "anoncvs" ];then
	# Checkout if CVS-User is anonymous
		cd "$CVSDIR"
		cvs -z3 $cvs_debug -d anoncvs@cvs.tuxbox.org:/cvs/tuxbox diff -U 3 -R >$MyPATCHESDIR/$complete_diff
	else
	# Checkout if CVS-User is a registred user
		cd "$CVSDIR"
		cvs -z3 $cvs_debug -d "$CVSNAME@cvs.tuxbox.org:/cvs/tuxbox"  diff -U 3 -R >$MyPATCHESDIR/$complete_diff
	fi
}

# -----------------------------------------------------------------------------------------------------------
# download and update yBuild
# -----------------------------------------------------------------------------------------------------------
do_yb_update()
{
	echo "=============================================================="
	echo "Start UPDATE yBuild"
	echo "make a copy of yb_globals.conf befor updating. Compare after"
	echo "update if there are new entries"
	echo "=============================================================="
	echo "looking for version file"
	echo "YOUR LOCAL version file"
	if [ -e ./yb_version.txt ]; then
		cat ./yb_version.txt
	else
		echo "does not exists"
	fi
	echo "REMOTE version file"
	wget -O - -q "http://www.yjogol.de/download/yb_version.txt"

	echo "=============================================================="
	echo "Download and update scripts? (y/n)"
	echo "=============================================================="
	read yesno
	if [ "$yesno" == "y" ];then
		wget -O ./yBuild.zip "http://www.yjogol.de/download/yBuild.zip"
		unzip ./yBuild.zip
		echo "!!!! you need to restart the script !!!"
		echo "Ready .... press Enter"
		read dummy
	fi
}

# -----------------------------------------------------------------------------------------------------------
# create yBuild distribution
# -----------------------------------------------------------------------------------------------------------
do_yb_distribution()
{
	cd $SCRIPTDIR
	echo "Build distribution"
	dist=distribution
	rm -r ./$dist
	mkdir ./$dist
	cp -r ./include ./$dist
	cp ./yb* ./$dist 
	cp ./read* ./$dist
	cp ./chan* ./$dist
	cp -r ./exampl* ./$dist
	cd ./$dist
	for i in `find -name CVS` `find -name "*~"`
	do
		rm -r $i
	done
	zip -r yBuild *
	cd ..
}

# -----------------------------------------------------------------------------------------------------------
# MAIN
# -----------------------------------------------------------------------------------------------------------
init_variables
doquit=false

# Main Menu
while [ "$doquit" == "false" ]
do
	menu_start
	read cmd
	
	case "$cmd" in 
		0)
			basisconf
			init_variables
			;;
		cc)
			configureconf
			init_variables
			;;
		bc)
			buildconf
			init_variables
			;;
		go)
			do_delete_files
			cvs_checkout_update
			do_configure_flash
			do_build $cBuildTARGET $cBuildGUI $cBuildFS $cBuildCHIPS
			;;
 		co)
			do_delete_files
			cvs_checkout_all
			echo "Ready .... press Enter"
			read dummy
			;;
		cu)
			do_delete_files
			cvs_checkout_update
			echo "Ready .... press Enter"
			read dummy
			;;
		cf)
			do_configure_flash
			echo "Ready .... press Enter"
			read dummy
			;;
		cy)
			do_configure_yadd
			echo "Ready .... press Enter"
			read dummy
			;;
		p)
			do_patch
			echo "Ready .... press Enter"
			read dummy
			;;
		bs)
			do_build $cBuildTARGET $cBuildGUI $cBuildFS $cBuildCHIPS
			echo "Ready .... press Enter"
			read dummy
			;;
		bf)
			do_build flash all all all
			echo "Ready .... press Enter"
			read dummy
			;;
		by)
			do_build yadd all
			echo "Ready .... press Enter"
			read dummy
			;;
		lc)
			cp $MyLOCALSDIR/*local.sh $CVSDIR/cdk
			echo "Ready .... press Enter"
			read dummy
			;;
		lb)
			cp $CVSDIR/cdk/*local.sh $MyLOCALSDIR/
			echo "Ready .... press Enter"
			read dummy
			;;
		cd)
			do_create_diff
			echo "Ready .... press Enter"
			read dummy
			;;
		ct)
			cleantargets
			;;
		mt)
			maketargets
			;;
		m)
			makeit
			;;
		t)
			$CVSDIR/hostapps/toolchecker/toolchecker.sh
			echo "Ready .... press Enter"
			read dummy
			;;
		d)
			if [ "$DEBUG" == "0" ]
			then
				DEBUG=1
			else
				DEBUG=0
			fi
			;;
		l)
			if [ -e $LOGFILE ]
			then
				more $LOGFILE
			else
				echo "no log file exists"
			fi
			echo "Ready .... press Enter"
			read dummy
			;;
		d0)
			cd /etc/init.d
			sudo /etc/init.d/dhcp3-server stop
			echo "Ready .... press Enter"
			read dummy
			;;
		d1)
			cd /etc/init.d
			sudo /etc/init.d/dhcp3-server start
			echo "Ready .... press Enter"
			read dummy
			;;
		x)
			doquit="true"
			;;
		yn)
			do_build_yadd
			echo "Ready .... press Enter"
			read dummy
			;;
		te)
		 	do_configure_build_features FLASH
		 	do_configure_build_features YADD
			echo "Ready .... press Enter"
			read dummy
			;;
		ccc)
			ccache -c
			echo "Ready .... press Enter"
			read dummy
			;;
		dis)
			do_yb_distribution
			echo "Ready .... press Enter"
			read dummy
			;;
		upd)
			do_yb_update
			;;
	esac
done
